#-- 
#  Copyright (C) 2008 Red Hat Inc.
#  
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#  
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#  
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
#
# Author: Bryan Kearney <bkearney@redhat.com>
#--

require "tempfile"
require "ace/util"
require "ace/yum_updater"

# An appliance configuration engine. This engine supports an appliance
# definition which is a puppet recipe. The engine can include
# modules which are re-usable puppet modules. 
class ACE	
    include Ace::Util
	
	# The name of the appliance to be created
	attr_accessor(:appliance_name)
	attr_accessor(:log_file)
	attr_accessor(:ace_directory)	
	attr_accessor(:ace_path)	
	attr_accessor(:debug)	
	attr_accessor(:verbose)	
	

	def initialize() 
		self.log_file= nil
		self.debug= false
		self.verbose= true 
		self.ace_directory= ENV["ACEHOME"].nil? ? "/usr/share/ace/" : ENV["ACEHOME"]	
		self.ace_path = ENV["ACEPATH"]
	end
	
	def install_appliance()	
		self.check_for_appliance()
		# Pre-install to bring up the network
		command = self.base_command()
		command += " #{File.join(self.ace_directory(), "manifests", "pre_install.pp")}"
		status1 = my_exec(command)		
		
		# Now do the acutal install
		command = self.base_command()
		command += " #{self.appliance_file}"
		status2 = my_exec(command)
        return status2 if status2 > status1 
        return status1
	end
	
	def refresh_appliance()
		return self.install_appliance	
	end	
	
	def update_appliance
        status = 0 
	    self.check_for_appliance()
	    package_updater = get_package_updater() ;
	    if package_updater.updates_pending?
	        log_info ("Updating appliance from #{appliance_dir}")  
	        package_updater.update()
	        # first, re-execute the standard recipe in case that was modified
	        status = install_appliance()
	        
	        # now, iterate over any new update recipies which may exist
	        update_recipies = self.find_all_in("/appliances/#{appliance_name}/update*","*.pp")
	        update_recipies.each do |file, path|
                command = self.base_command()
                command += " #{path}"      
	            new_status = my_exec(command)
                status = new_status if new_status > status
            end
            command = self.base_command()
            command += " #{File.join(self.ace_directory(), "manifests", "post_update.pp")}"
            new_staus = my_exec(command)
            status = new_status if new_status > status
	    else
	        log_info("No updates necessary")     
        end
        return status
    end
	
	def generate_graph()	
		self.check_for_appliance()	
		command = self.base_command()
		command += " --graphdir=#{ENV["PWD"]}"
		command += " --graph"
		command += " #{self.appliance_file}"
		return my_exec(command)	
	end	
	
	def parse()	
		self.check_for_appliance()	
		command = self.base_command()
		command += " --parseonly"
		command += " #{self.appliance_file}"
        return my_exec(command)	
	end	
	
	def list_appliances
		items = find_all_in("/appliances/*", nil) 
		items.each do |key,value|
			puts("#{key} (#{value})")
		end
        return 0
	end 

	def list_modules
		items = find_all_in("/modules/*",nil) 
		items.each do |key,value|
			puts("#{key} (#{value})")
		end
        return 0
	end

	def list_templates
		items = find_all_in("/*/*/templates","*.erb") 
		items.each do |key,value|
			puts("#{key} (#{value})")
		end
        return 0
	end 
	
	def list_single_execs
		items = find_all_in("/var/ace/single_execs","*") 
		items.each do |key,value|
			puts("#{key} (#{value})")
		end
        return 0
	end 	

	def list_resources
		items = find_all_in("/*/*/files","*") 
		items.each do |key,value|
			puts("#{key} (#{value})")
		end
        return 0
	end 
	
	def list_updates
		items = find_all_in("/appliances/*/update*","*.pp") 
		items.each do |key,value|
			puts("#{key} (#{value})")
		end
        return 0
	end 	
	
	def log_info(string)
        log_msg(string) if debug || verbose
	end
	
	def log_debug(string)
	    log_msg(string) if debug
    end
    
    def log_msg (string)
        if (log_file.nil?)
            puts(string)
        else
            logfile = File.open(log_file, "a")
            logfile.puts("%s %s" % [Time.now(), string])
            logfile.close()
        end    
    end	

	#protected
	
	def appliance_dir
		name, location = find_all_in(File.join("/appliances", self.appliance_name), nil).first
		return location
	end	
	
	def full_path_array
	    path = ""
	    if !self.ace_path.nil?
	    	path << self.ace_path << ":"
	    end
		path << self.ace_directory
		return path.split(":")
	end
	
	def appliance_file
		return File.join(self.appliance_dir(), "#{self.appliance_name()}.pp")
	end
	
	def module_path
		module_path_array = self.full_path_array.collect { |path| File.join(path, "modules")}
		return module_path_array.join(":") 
	end
	
	def extended_module_path
		module_path = self.module_path() << ":" << File.dirname(self.appliance_dir())
		return module_path
	end

    # This is used to call puppet. It therefore tries to modify the
    # wacky puppet return codes. Basically, if hte return code is
    # greater then one then the third bit denotes an error.
    # http://projects.reductivelabs.com/issues/show/1633
	def my_exec(cmd, user=nil, group=nil)
	    self.log_debug("executing #{cmd}")	
		output, status = clean_exec(cmd, user, group, !verbose)
        if status > 1
            status = status[2]
        end
        return status
    end
    
    def get_package_updater()     
        return Ace::YumUpdater.new()
    end
	
	def base_command() 
		command = "puppet"
		command += " -d" if self.debug()
		command += " -v" if self.verbose()
        command += " --detailed-exitcodes"
		command += " --templatedir=#{File.join(self.appliance_dir(), 'templates')}"
		command += " --modulepath=#{self.extended_module_path}"
		command += " --factpath=#{File.join(self.ace_directory(), "facts")}"
		command += " --logdest=#{self.log_file}" if !self.log_file.nil?
		
		return command	
	end	
	
	def check_for_appliance
		if self.appliance_dir.nil?
			puts ("Appliance #{self.appliance_name} not found")
			exit(1)
		end
	end


	# Locate all files, the complete path, with a given extension
	def find_all_in(dir, type="*.rb")
		items = Array.new()
	    self.full_path_array().each do |item|
			ldir = File.join(item, dir)
			ldir = File.join(ldir, type) if type != nil
			Dir.glob(ldir) do |file|
				base = File.basename(file)
				items << [base, file]
			end
		end
		return items
	end

	def find_file(file)
		items = find_all_in(file, nil)
		return items.values[0] if items.size == 1
	end
	
	def append_if_not_found(file, string)
        if (!File.exist?(file)) || (! File.read(file).include?(string) )
            file = File.open(file, "a")
            file.puts(string)		
        end 
    end			
end


