class firewall {

    $firewall_dir = "/usr/share/firewall"
    
    File {
        owner   => root,
        group   => root,
        mode    => 0644,
    }

    file {"/etc/sysconfig/iptables" :
        ensure => present
    }
    
    service { "firewall":
        name            => "iptables",
        ensure          => running,
        enable          => true,
        hasstatus       => true,
        require         => File["/etc/sysconfig/iptables"]
    }

    # the reload script (thanks rmonk)
    file { "iptables-update":
        name    => "/usr/local/bin/iptables-update.sh",
        mode    => 0755,
        source  => "puppet:///firewall/iptables-update.sh",
        notify  => Exec["reload-firewall"],
    }
    
    file { "iptables-noop":
        name    => "/usr/local/bin/iptables-noop.sh",
        mode    => 0755,
        source  => "puppet:///firewall/iptables-noop.sh",
    }    

    # setup directories
    file { [ $firewall_dir, "$firewall_dir/filter", "$firewall_dir/nat", "$firewall_dir/mangle" ]:
        ensure          => directory,
        mode            => 0755,
        notify          => Exec["reload-firewall"],
    }

    file { [ "$firewall_dir/filter/INPUT", "$firewall_dir/filter/OUTPUT", "$firewall_dir/filter/FORWARD" ]:
        ensure          => directory,
        mode            => 0755,
        require         => File["${firewall_dir}/filter"],
        notify          => Exec["reload-firewall"],
    }
    
    file {"$firewall_dir/nat/POSTROUTING":
        ensure          => directory,
        mode            => 0755,
        require         =>  File["${firewall_dir}/nat"],
        notify          => Exec["reload-firewall"],
    }    

    # real files
    file { "$firewall_dir/filter/INPUT.head":
        ensure  => present,
        source  => "puppet:///firewall/INPUT.head",
        notify  => Exec["reload-firewall"],
    }
    file { "$firewall_dir/filter/INPUT.tail":
        ensure  => present,
        source  => "puppet:///firewall/INPUT.tail",
        notify  => Exec["reload-firewall"],
    }
    file { "$firewall_dir/filter/OUTPUT.head":
        ensure  => present,
        source  => "puppet:///firewall/OUTPUT.head",
        notify  => Exec["reload-firewall"],
    }
    file { "$firewall_dir/filter/OUTPUT.tail":
        ensure  => present,
        source  => "puppet:///firewall/OUTPUT.tail",
        notify  => Exec["reload-firewall"],
    }
    
    file { "$firewall_dir/nat/POSTROUTING.head":
        ensure  => present,
        source  => "puppet:///firewall/OUTPUT.tail",
        notify  => Exec["reload-firewall"],
    }    

    # relevent execs
    exec { "reload-firewall":
        command         => "/usr/local/bin/iptables-update.sh",
        require         => File["iptables-update"],
        refreshonly     => true,
    }
    
    exec { "save-config":
    	command			=> "/sbin/iptables-save > /etc/sysconfig/iptables",
    	refreshonly		=> true,
    	require 		=> Exec["reload-firewall"]
    }
    	
}

class firewall::disabled inherits firewall {
    
    Service["firewall"] {
        name            => "iptables",
        ensure          => stopped,
        enable          => false,
        hasstatus       => true,
        require         => File["/etc/sysconfig/iptables"]        
    }
    
    # Make these execs be no-ops
    Exec ["reload-firewall"] {
        command         => "/usr/local/bin/iptables-noop.sh",
        require         => File["iptables-noop"],
        refreshonly     => true,
    }
    
    Exec ["save-config"] {
    	command			=> "/usr/local/bin/iptables-noop.sh",
    	refreshonly		=> true,
    	require 		=> Exec["reload-firewall"]
    }    
}

# usage
# firewall::rule { 'rulename':
#                     chain  => "INPUT",
#                     srcprt => 123423,
#                     dstprt => 22,
#                     src    => foo.com,
#                     dst    => bar.com,
#                     action => ACCEPT
#                 }
define firewall_rule (
    $chain = 'INPUT',
    $comment = '',
    $protocol = 'tcp',
    $source_port = '',
    $destination_port = '',
    $source = '',
    $destination = '',
    $action = 'ACCEPT',
    $table = 'filter',
    $out_interface = '',
    $in_interface = ''
    ) {
    file { "/usr/share/firewall/${table}/${chain}/${name}":
        owner           => root,
        group           => root,
        ensure          => present,
        content         => template("firewall/rule.erb"),
        require         => [ File["/usr/share/firewall/${table}/${chain}"], Service["firewall"] ],
        notify          => [ Exec["reload-firewall"], Exec["save-config"] ]
    }
}

define firewall::setup($status = "enabled")
{
    case $status {
        enabled:   { include firewall } 
        disabled:  { include firewall::disabled  } 
        default:   { include firewall } 
    }
}

define firewall_open_port($port = "80", $policy = "tcp")
{
	include firewall
	
	firewall_rule{"$name": destination_port=>$port, protocol => $policy}
}
